# DocumenterInterLinks.jl

[![Version](https://juliahub.com/docs/DocumenterInterLinks/version.svg)](https://juliahub.com/ui/Packages/General/DocumenterInterLinks)
[![Stable](https://img.shields.io/badge/docs-stable-blue.svg)](https://juliadocs.org/DocumenterInterLinks.jl/)
[![Dev](https://img.shields.io/badge/docs-dev-blue.svg)](https://juliadocs.org/DocumenterInterLinks.jl/dev)
[![Build Status](https://github.com/JuliaDocs/DocumenterInterLinks.jl/workflows/CI/badge.svg)](https://github.com/JuliaDocs/DocumenterInterLinks.jl/actions)
[![Coverage](https://codecov.io/gh/JuliaDocs/DocumenterInterLinks.jl/branch/master/graph/badge.svg)](https://codecov.io/gh/JuliaDocs/DocumenterInterLinks.jl)

A plugin for [Documenter.jl](https://documenter.juliadocs.org/) that enables linking between projects.

The package provides an [`InterLinks`](https://juliadocs.org/DocumenterInterLinks.jl/stable/internals/#DocumenterInterLinks.InterLinks) plugin that loads [inventory files](#inventories) from other project documentations build with [Documenter](https://documenter.juliadocs.org/) or [Sphinx](https://www.sphinx-doc.org/en/master/). It then extends Documenter's markdown link syntax to recognize an `@extref` link target and resolve it with a link to the external project (similarly to the built-in `@ref` links for internal links), see [External References](#external-references).


## Installation

As usual, the package can be installed via

```
] add DocumenterInterLinks
```

in the Julia REPL, or by adding

```
DocumenterInterLinks = "d12716ef-a0f6-4df4-a9f1-a5a34e75c656"
```

to the relevant `Project.toml` file (e.g., `docs/Project.toml`).


## Plugin Instantiation

In `docs/make.jl`, instantiate an [`InterLinks`](https://juliadocs.org/DocumenterInterLinks.jl/stable/internals/#DocumenterInterLinks.InterLinks) object, e.g.,

```julia
using DocumenterInterLinks

links = InterLinks(
    "matplotlib" => "https://matplotlib.org/3.7.3/",
    "sphinx" => "https://www.sphinx-doc.org/en/master/",
)
```

See [`docs/make.jl`](https://github.com/JuliaDocs/DocumenterInterLinks.jl/blob/master/docs/make.jl#L11) for another example.

The resulting plugin object must be passed as an element of the `plugins` keyword argument to `Documenter.makedocs`. This then enables `@extref` links in the project's documentation to be resolved.

See [`docs/src/inventories`](https://github.com/JuliaDocs/DocumenterInterLinks.jl/tree/master/docs/src/inventories) for some exemplary inventory files in the [TOML format](https://juliadocs.org/DocInventories.jl/stable/formats/#TOML-Format).


## External References

Instead of [Documenter's `@ref`](https://documenter.juliadocs.org/stable/man/syntax/#@ref-link), the `@extref` link target can be used to resolve the link via any of the available projects defined in the `InterLinks` plugin:

```
* [`Documenter.makedocs`](@extref)
* [Documenter's `makedocs` function](@extref `Documenter.makedocs`)
* See the section about Documenter's [Writers](@extref Documenter).
```

See the [documentation](https://juliadocs.org/DocumenterInterLinks.jl/dev/#Using-External-References) and [recommended syntax](https://juliadocs.org/DocumenterInterLinks.jl/dev/syntax/#Recommended-Syntax) for details.


## Inventories

Inventory files (`objects.inv`) are automatically generated by [Documenter.jl](https://documenter.juliadocs.org/stable/) ≥ `v1.3.0` and [Sphinx](https://www.sphinx-doc.org/en/master/). For projects using `Documenter v0.25.0`–`v1.2.0` , the [`DocumenterInventoryWritingBackport` package](https://github.com/JuliaDocs/DocumenterInventoryWritingBackport.jl) provides a backport of the inventory-writing functionality. For these projects, simply loading `DocumenterInventoryWritingBackport` in the `docs/make.jl` file will ensure the creation of a `objects.inv` file.

For some projects that do not (or not yet) provide an inventory, manually generated inventory files in human-readable [`.toml` format](https://juliadocs.org/DocInventories.jl/stable/formats/#TOML-Format) are collected in the [Inventory File Repository (Wiki)](https://github.com/JuliaDocs/DocumenterInterLinks.jl/wiki/Inventory-File-Repository). Feel free to [generate your own inventory files](http://juliadocs.org/DocumenterInterLinks.jl/stable/howtos/#howto-manual-inventory) and contribute them. These `.toml` files can be copied to the source of your project's documentation, and loaded in the `InterLinks` plugin as local sources of inventory information.


## Documentation

The [full documentation of this project is available online](https://juliadocs.org/DocumenterInterLinks.jl/dev/).


## Related Projects

* [DocInventories.jl](http://juliadocs.org/DocInventories.jl/stable/) – The backend used for processing inventory files, e.g., the `objects.inv` file generated by [Documenter.jl](https://documenter.juliadocs.org/stable/) and [Sphinx](https://www.sphinx-doc.org/en/master/).
* The [Intersphinx plugin](https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html) for [Sphinx](https://www.sphinx-doc.org/en/master/) – The Python-equivalent of this project.
